<?php
namespace Teknisa\MAN\Controllers;

use Zeedhi\Framework\Controller\CrudWithParameterBag;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Response\Notification;
use Teknisa\MAN\Utilities\Util;
use Teknisa\MAN\Services;
use \Exception as Exception;

class Tag extends CrudWithParameterBag {

    protected $dataSourceNameArray = array(
        'dataTag'          => 'select_tag',
        'dataUnidadeTag'   => 'select_unidadeTag',
        'dataProdutoTag'   => 'select_produtoTag',
        'produtoFilialTag' => 'select_productfilialtag'
    );
    
    protected $util,
              $environment;

    public function __construct(Services\TagService $TagService, Manager $dataSourceManager, ParameterBag $parameterBag, Util $util, $environment) {
        $this->TagService                       = $TagService;
        $this->parameterBag                     = $parameterBag;
        $this->dataSourceManager                = $dataSourceManager;
        $this->util                             = $util;
        $this->environment                      = $environment;
    }
    
    public function findTag(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameArray['dataTag'];
        $this->parameterBagColumns = array('NRORG');
        parent::find($request, $response);
    }
    
    public function saveTag(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $dataSet = $request->getDataSet();
            $dataSet->setDataSourceName($this->dataSourceNameArray['dataTag']);
            $persistedRows = $this->dataSourceManager->persist($dataSet);
            $response->addDataSet(new DataSet($this->dataSourceNameArray['dataTag'], $persistedRows));
            
        } catch (\Exception $e) {
            $response->setCriticalError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }
    
    public function deleteTagCascade(DTO\Request\DataSet $request,DTO\Response $response) {
    	try {
    	    $this->dataSourceName = $this->dataSourceNameArray['dataTag'];
    	    $params = $this->util->getParamsAsArray($request);
    	    $params['NRORG'] = $this->environment->getNrOrgTrab();
            $this->TagService->deleteTagCascade($params[0]['CDTAG'], $params['NRORG']);
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }
    

    public function getNovoCodigo(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        $params = array(
            ':NRORG' => $row['NRORG']
        );
        $cdsNovoCodigo = $this->TagService->getNovoCodigo($params);
        
        print str_pad($cdsNovoCodigo[0]['NEXT_CD'], 4, "0", STR_PAD_LEFT);

    }
    
    //Manipulação das Unidades
    public function findUnidadeTag(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameArray['dataUnidadeTag'];
        $this->parameterBagColumns = array('NRORG','CDTAG');
        parent::find($request, $response);
    }
    
    public function saveUnidadeTag(DTO\Request\DataSet $request, DTO\Response $response){
        $this->dataSourceName = $this->dataSourceNameArray['dataUnidadeTag'];
        $dataSet = $request->getDataSet();
        $rows = $dataSet->getRows();
        
        try {
            $dataSet = $request->getDataSet();
            $dataSet->setDataSourceName($this->dataSourceNameArray['dataUnidadeTag']);
            
            $validacao = $this->TagService->getUnidadesRepetidas($rows[0]);
            
            if(count($validacao) > 0) {
                throw new Exception("Filial já cadastrada!");
            }
            
            $persistedRows = $this->dataSourceManager->persist($dataSet);
            
            $response->addDataSet(new DataSet($this->dataSourceNameArray['dataUnidadeTag'], $persistedRows));
            $nrPersistedRows = count($persistedRows);
            
        } catch (Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function deleteUnidadeTagCascade(DTO\Request\DataSet $request,DTO\Response $response) {
    	try {
    	    $this->dataSourceName = $this->dataSourceNameArray['dataUnidadeTag'];
    	    $params = $this->util->getParamsAsArray($request);
    	    $params['NRORG'] = $this->environment->getNrOrgTrab();
            $this->TagService->deleteUnidadeTagCascade($params[0]['CDTAG'], $params['NRORG'],$params[0]['CDFILIAL']);
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }
    
    public function deleteUnidadeTag(DTO\Request\DataSet $request,DTO\Response $response) {
        try {
    	    $this->dataSourceName = $this->dataSourceNameArray['dataUnidadeTag'];
            parent::delete($request,$response);
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }
    
    
    
    //Manipulação dos Produtos
    
    public function findProdutosFilial(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameArray['produtoFilialTag'];
        $this->parameterBagColumns = array('NRORG','CDFILIAL');
        parent::find($request, $response);
    }
    
    public function findProdutoTag(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameArray['dataProdutoTag'];
        $this->parameterBagColumns = array('NRORG','CDFILIAL','CDTAG');
        parent::find($request, $response);
        
    }
    
    public function saveProdutoTag(DTO\Request\DataSet $request, DTO\Response $response){
        $this->dataSourceName = $this->dataSourceNameArray['dataProdutoTag'];
        
        try {
            foreach ($request->getDataSet()->getRows() as $row) {
                $arrayCodigoProduto = $row['CDPRODUTO'];
                
                foreach ($arrayCodigoProduto as $produto) {
                    $row['CDPRODUTO'] = $produto;
                    $row['__is_new'] = true;
                    $row['changed'] = false;
                    $this->dataSourceManager->persist(new DataSet($this->dataSourceNameArray['dataProdutoTag'], array($row)));
                }
            }
            
            $response->addNotification(new Notification("Dados inseridos com sucesso!", Notification::TYPE_SUCCESS));
        } catch (Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function deleteProdutoTag(DTO\Request\DataSet $request,DTO\Response $response) {
        try {
    	    $this->dataSourceName = $this->dataSourceNameArray['dataProdutoTag'];
            parent::delete($request,$response);
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }
    
}    